/**
 *	RELATION DATA ARC-BC MINER 
 *  --------------------------
 *  author: Roman Kolar
 *  login: xkolar37 
 *  year: 2008
 *  --------------------------
 * 
 */

package textclassifier;
import java.sql.*;
import java.sql.Types;
import java.io.*;
import textclassifier.database.*;
import textclassifier.ARCBC.*;
import textclassifier.mining.*;

import java.util.*;
import java.io.*;
/**
 * Main class used to run the classifier. It performs processing of command line attributes
 * and configuration file. Then, connection to MySQL is establisjed and classification 
 * method is started.
 *
 * @param  DB_SERVER  URL of MySQL server where the input table is located
 * @param  DB_DATABASE Name of a database on a MySQL server
 * @param  DB_USER  MySQL Database user 
 * @param  DB_USER  MySQL Database user 
 * @param  tablename    Name of the input table located on a server
 * @param  inatts   Set of input attributes of classification
 * @param  discatts   set of input attibutes to be discretized
 * @param  outatt set of input attibutes to be discretized
 * @param   min_supp    minimum support threshold
 * @param   min_conf    minimum confidence threshold
 * @param   dbl     Instance of an assocation rule based classifier
 */

public class Main{

	/**
	 * Server name 
	 */
	private String DB_SERVER;
	
	/**
	 * Database name 
	 */
	private String DB_DATABASE;
	
	/**
	 * User name 
	 */
	private String DB_USER;
	
	/**
	 * User password 
	 */
	private String DB_PASS;

	/**
	 * Default support 
	 */
        
        private String tablename;
        private ArrayList<String> inatts;   // set of input attributes of classification
        private ArrayList<String> discatts; // set of input attibutes to be discretized
        private String outatt;              // attribute, which is classified
	double min_supp = 0.05;             //min support of f. itemset
		
	/**
	 * Default confidence 
	 */
	double min_conf = 0.6; //min confidence of AR
	
	ArcBC dbl;          // classifier

/**
 * Processes one line of the configuration file and stores the information in 
 * variables of the class.
 *
 * @param  str  string containing input from the file
*/
      private void processString (String str)
        {
            if (str.startsWith("server=")) {
                DB_SERVER = str.substring(7);
            }
            else if (str.startsWith("database=")) {
                DB_DATABASE = str.substring(9);
            }
            else if (str.startsWith("user=")) {
                DB_USER = str.substring(5);
            }
            else if (str.startsWith("password=")) {
                DB_PASS = str.substring(9);
            }
            else if (str.startsWith("output=")) {
                outatt = str.substring(7);
            }
            else if (str.startsWith("tablename=")) {
                tablename = str.substring(10);
            }
            else if (str.startsWith("input=")) {
               inatts.add(str.substring(6));
            }
            else if (str.startsWith("inputdisc=")) {
               inatts.add(str.substring(10));
               discatts.add(str.substring(10));
            }
        }
        
 /**
 * Method used to start methods from other classes in order to perform required functionality
 * of associaion rule based classifier
 * 
 * @param  supp  value of minimum support threshold (0-1)
 * @param  conf  value of minimum confidence threshold (0-1)
 * @param  inputfile  path to the configuration file 
 */
     
	public Main(double supp, double conf, String inputfile){
   
            /* Config file reading */
                BufferedReader in;
                inatts = new ArrayList<String>();
                discatts = new ArrayList<String>();
                try {
                    in = new BufferedReader(new FileReader(inputfile));
                    String str;
                    while ((str = in.readLine()) != null) {
                        processString(str);
                    }
                    in.close();
                } catch (java.io.FileNotFoundException ee) {
                                System.err.println("Config file not found");
    				System.exit(0);
                } catch (IOException e) {
                                System.err.println("input/output exception");
                                System.exit(1);
                }
        
            
            /* Check of minsupp and minconf */
                if(conf<0.0 || supp < 0.0) System.exit(0);
		if(conf>1.0 || supp > 1.0) System.exit(0);		
		min_supp = supp;
		min_conf = conf;

		int lrgitemset_level = 0;
		int gr1_index, gr2_index;

		ArrayList<ArrayList<largeItemset>> allLargeItemsets;				
		ArrayList<AssociationRule> allRules = new ArrayList<AssociationRule>();
		Category categories[];
             /* Connecting to the database */
		MySQLAccess dbaccess;
		dbaccess = new MySQLAccess(DB_SERVER,DB_DATABASE,DB_USER,DB_PASS);
		dbaccess.makeConnection();
	
		dbl = new ArcBC(new OstraDBLoaderBC(outatt, tablename, inatts, discatts),dbaccess);		
		System.out.println("Min. support = "+min_supp);
	  

              /* training phase */
		allRules = dbl.train((float)min_supp,(float)min_conf);	
		System.out.println("Total rules found in all categories: "+allRules.size());

		int totalok=0;
		int totalnon=0;	
              /* classification phase */
		ArrayList<Document> allDocs = dbl.getAllDocuments();
		System.out.println("CELKOVE DOKUMENTU: "+allDocs.size());
		int res =0;
		for(int i =0; i<allDocs.size(); i++){
			res = dbl.classifyDocument(allRules,allDocs.get(i),0.5);			
			
			if(res==-1) totalnon++;
			if(res==1)totalok++;
			//System.out.println(allDocs.get(i).toString());
		}
		
		int total = allDocs.size();
		System.out.println("==========================================================");
		System.out.println("Total documents: "+total);
		System.out.println("Non classified: "+totalnon);
		System.out.println("Classified: "+(total-totalnon));
		System.out.println("==========================================================");		
		System.out.println("Well classified: "+totalok);
		System.out.println("OK %: "+(double)totalok/(double)total);
		System.out.println("MR %: "+(1 - (double)totalok/(double)total));		
		System.out.println("TOTAL/NON OK %: "+(double)(totalok)/(double)(total-totalnon));
		System.out.println("==========================================================");				                   
	}
        
 /**
 * The main method of the program. It is checking the command line parameters and
 * strarts the Main function. 
 * 
 * @param  argsv  command line arguments - min. support, min. confidence, configuration file
 */	
        
	public static void main(String argsv[]){

        if(argsv.length==3){
		Double supp = -1d;
		Double conf = -1d;
                  String s = argsv[0];
                  String c = argsv[1];           
				try {
  					 supp = Double.parseDouble(s);
  					 conf = Double.parseDouble(c);
				}
				catch (NumberFormatException exc) {
	    			// It's not a valid double.
    				System.err.println("Illegal number format");
    				System.exit(0);
				}
				Main m = new Main(supp,conf, argsv[2]);

        }else{
		System.out.println("Usage:");
		System.out.println("textclassifier <min_supp> <min_conf>");
	}
	
	}
}
